import sys
import os

from os import environ as env
from os.path import isdir, isfile, abspath, dirname, basename

from envbash import load_envbash
from IzzyLib import logging
from IzzyLib.misc import boolean

VERSION = '0.3.0b'

full_path = abspath(sys.executable) if getattr(sys, 'frozen', False) else abspath(__file__)
script_path = getattr(sys, '_MEIPASS', dirname(abspath(__file__)))
script_name = basename(full_path)
stor_path = abspath(f'{os.getcwd()}/data')


if not isdir(stor_path):
	os.makedirs(stor_path, exist_ok=True)


if not boolean(env.get('LOGDATE', 'yes').lower()):
	logging.setConfig({'date': False})


if not isfile(f'{stor_path}/production.env'):
	logging.error(f'PAWS environment file doesn\'t exist: {stor_path}/production.env')
	logging.info('Creating a new config file. Be sure to edit it and restart PAWS')

	new_config = '''###
# Uncomment and adjust any values as necessary
###

### Port and host PAWS will listen on
#PAWS_HOST=127.0.0.1
#PAWS_PORT=3001

#PAWS_DOMAIN=bappypaws.example.com
#PAWS_NORSS=true

### Require approval for unknown instances. They can be accepted or denied at {{MASTODOMAIN}}/paws/list/requests
#PAWS_REQ_APPROVAL=false

### Override domain in mastodon's config
#MASTODOMAIN=mastodon.example.com

### 
#MASTOPATH=/home/mastodon/glitch-soc
#MASTOHOST=localhost:3000
'''

	with open(f'{stor_path}/production.env', 'w') as pawsconf:
		pawsconf.write(new_config)

	logging.info(f'Created new config at {stor_path}/production.env')

else:
	load_envbash(f'{stor_path}/production.env')


req_approval = boolean(env.get('PAWS_REQ_APPROVAL'), False)


PAWSCONFIG = {
	'host': env.get('PAWS_HOST', '127.0.0.1'),
	'port': int(env.get('PAWS_PORT', 3001)),
	'domain': env.get('PAWS_DOMAIN', 'bappypaws.example.com'),
	'disable_rss': boolean(env.get('PAWS_DISABLE_RSS', True)),
	'require_approval': req_approval,
	'mastopath': env.get('MASTOPATH', os.getcwd()),
	'mastohost': env.get('MASTOHOST', 'localhost:3000'),
}


masto_path = PAWSCONFIG['mastopath']

if not isfile(f'{masto_path}/.env.production'):
	logging.error(f'Mastodon environment file doesn\'t exist: {masto_path}/.env.production')

else:
	load_envbash(f'{masto_path}/.env.production')

MASTOCONFIG={
	'domain': env.get('MASTODOMAIN', env.get('WEB_DOMAIN', env.get('LOCAL_DOMAIN', 'localhost:3000'))),
	'auth_fetch': boolean(env.get('AUTHORIZED_FETCH')),
	'dbhost': env.get('DB_HOST', '/var/run/postgresql'),
	'dbport': int(env.get('DB_PORT', 5432)),
	'dbname': env.get('DB_NAME', 'mastodon_production'),
	'dbuser': env.get('DB_USER', env.get('USER')),
	'dbpass': env.get('DB_PASS')
}
