import os
import json
import sys
import time
import traceback

import http.client as http

from IzzyLib import logging
from urllib.parse import urlencode, urlparse
from mastodon import Mastodon

from .config import PAWSCONFIG, MASTOCONFIG, VERSION, stor_path


HEADERS = {
	'User-Agent': f'PAWS/{VERSION}',
	'Accept': 'application/json'
}
SCOPES = ['read:accounts']
REDIR_URI = f'https://{MASTOCONFIG["domain"]}/paws/auth'
WEBSITE = 'https://git.barkshark.xyz/izaliamae/paws'


# some instances use a different domain for federation
def get_webhost(domain):
	try:
		conn = http.HTTPSConnection(domain)
		conn.request('GET', '/.well-known/host-meta', headers=HEADERS)
		response = conn.getresponse()

		if response.status == 301:
			url = response.msg.get('Location')
			return urlparse(url).netloc

		elif response.status != 200:
			return ('error', f'Failed to fetch host-meta: received error {response.status}', 'heck')

	except:
		msg = f'Failed to connect to {domain}'
		logging.error(msg)
		return ('error', msg, 'heck')

	return domain


def create_app(domain):
	instance = get_webhost(domain)

	if 'error' in instance:
		return instance

	try:
		app_id, app_secret = Mastodon.create_app(
			f'PAWS @ {MASTOCONFIG["domain"]}',
			api_base_url=instance,
			scopes=SCOPES,
			redirect_uris=REDIR_URI,
			website=WEBSITE
		)
	except Exception as e:
		traceback.print_exc()
		logging.error(f'Failed to create app: {e}')
		return ('error', 'Failed to create app', 'heck')

	try:
		client = Mastodon(
			api_base_url=instance,
			client_id=app_id,
			client_secret=app_secret
		)

	except Exception as e:
		traceback.print_exc()
		logging.error(f'Failed to create app: {e}')
		return ('error', 'Failed to create app', 'heck')

	return (app_id, app_secret, client.auth_request_url(redirect_uris=REDIR_URI, scopes=SCOPES))


def login(user, code):
	fetch = Mastodon(
		api_base_url=user['domain'],
		client_id=user['appid'],
		client_secret=user['appsecret']
	)

	try:
		token = fetch.log_in(redirect_uri=REDIR_URI, scopes=SCOPES, code=code)

	except:
		msg = 'Failed to fetch token'
		logging.error(msg)
		return ('error', msg)

	try:
		client = Mastodon(api_base_url=user['domain'], access_token=token)
		fetch_user = client.me()

	except:
		msg = 'Failed to get user info'
		logging.error(msg)
		return ('error', msg)

	return (token, fetch_user)
